@@ -29,6 +29,7 @@
 #include <QTimer>
 #include <QSettings>
 #include <QObject>
+#include <QRegularExpression>
 #include <QFileDialog>
 
 MainWindow::MainWindow(QWidget *parent)
@@ -57,8 +58,13 @@ MainWindow::MainWindow(QWidget *parent)
     // This connection cares for displaying the XRechnung.
     QObject::connect(&_xrc, &XRControl::showXRechnung, this, &MainWindow::slotShowXRechnung);
 
+    ui->actionSaveAs->setEnabled(false);
+    ui->actionExportAsPdf->setEnabled(false);
+
     // connect menu items
     QObject::connect(ui->actionLoad, &QAction::triggered, this, &MainWindow::slotLoadFile);
+    QObject::connect(ui->actionSaveAs, &QAction::triggered, this, &MainWindow::slotSaveAs);
+    QObject::connect(ui->actionExportAsPdf, &QAction::triggered, this, &MainWindow::slotExportAsPdf);
     QObject::connect(ui->actionExit, &QAction::triggered, QCoreApplication::instance(), &QCoreApplication::quit);
     QObject::connect(ui->actionAbout_XRView, &QAction::triggered, this, &MainWindow::about);
 
@@ -162,6 +168,59 @@ void MainWindow::slotLoadFile()
     _files.append(urls);
 }
 
+void MainWindow::slotExportAsPdf()
+{
+    XRWidget *item = qobject_cast<XRWidget*>(ui->_mainToolBox->currentWidget());
+    if (item != nullptr) {
+        XRechnung *xr = item->xrechnung();
+        QString pdfFilePath = QFileDialog::getSaveFileName(
+            this,
+            tr("Export XRechnung as PDF"),
+            /// Remove file extension (if any) and append ".pdf" for suggested file name
+            xr->url().path().remove(QRegularExpression("\\.[^.]+$"))+".pdf",
+            tr("PDF (*.pdf)")
+        );
+        if (pdfFilePath.isEmpty()) { return; }
+        xr->exportPdf(pdfFilePath);
+    }
+}
+
+void MainWindow::slotSaveAs()
+{
+    XRWidget *item = qobject_cast<XRWidget*>(ui->_mainToolBox->currentWidget());
+    if (item != nullptr) {
+        XRechnung *xr = item->xrechnung();
+        QString asFilePath = QFileDialog::getSaveFileName(
+            this,
+            tr("Save XRechnung as"),
+            xr->url().path(),
+            tr("XML (*.xml)")
+        );
+        if (asFilePath.isEmpty()) { return; }
+        QString oldPath = xr->url().path();
+        // Don't save under same name
+        if (oldPath == asFilePath) { return; }
+        // Make sure the target file doesn't exist (Save as is an overwrite)
+        if (QFile::exists(asFilePath)) {
+            QFile::remove(asFilePath);
+        }
+        if (QFile::copy(oldPath, asFilePath)) {
+	        statusBar()->showMessage(tr("XRechnung sucessfully saved to given path."));
+        } else {
+	        QMessageBox::critical(
+		        this,
+		        tr("Saving XRechnung failed."),
+		        tr(
+			        "Could not save XRechnung to specified path.\n"
+			        "Possible causes are:\n"
+			        "  * You don't have the necessary permissions.\n"
+			        "  * The target file system is full."
+		        )
+	        );
+        }
+    }
+}
+
 void MainWindow::slotShowFileList()
 {
     int cnt = ui->_mainToolBox->count();
@@ -183,6 +242,8 @@ void MainWindow::slotShowXRechnung(XRechnung *xr)
 {
     if (xr == nullptr) return;
 
+    ui->actionSaveAs->setEnabled(false);
+    ui->actionExportAsPdf->setEnabled(false);
     ui->_mainToolBox->blockSignals(true);
     int indx = ui->_mainToolBox->addItem(new XRWidget(xr, this), xr->fileName());
     qDebug() << "Added index:" << indx;
@@ -192,6 +253,8 @@ void MainWindow::slotShowXRechnung(XRechnung *xr)
     });
     ui->_mainToolBox->setCurrentIndex(indx);
     ui->_mainToolBox->blockSignals(false);
+    ui->actionSaveAs->setEnabled(true);
+    ui->actionExportAsPdf->setEnabled(true);
 }
 
 void MainWindow::slotShowDocumentNo(int newIndx)
