@@ -29,6 +29,7 @@
 QString XRechnung::SaxonJar = QStringLiteral("saxon/jar");
 QString XRechnung::SaxonUbl = QStringLiteral("saxon/xslUbl");
 QString XRechnung::SaxonHtml = QStringLiteral("saxon/xslHtml");
+QString XRechnung::SaxonPdf = QStringLiteral("saxon/xslPdf");
 QString XRechnung::SaxonXsl = QStringLiteral("saxon/xsl");
 
 namespace {
@@ -67,6 +68,11 @@ QString XRechnung::html() const
     return _html;
 }
 
+QUrl XRechnung::url() const
+{
+    return _url;
+}
+
 QString XRechnung::value(const QString& name) const
 {
     QDomNodeList elems = _domDoc.elementsByTagName(name);
@@ -182,6 +188,63 @@ void XRechnung::slotHtmlFinished(int /*exitCode*/, QProcess::ExitStatus /*exitSt
     Q_EMIT htmlAvailable();
 }
 
+void XRechnung::exportPdf(const QString& path)
+{
+    QString file{ _url.toLocalFile() };
+    QSettings config;
+
+    const QString saxJar = config.value(SaxonJar).toString();
+    const QString xslPdf = config.value(SaxonPdf).toString();
+
+    const QStringList xsl_args {
+        "-jar",
+        saxJar,
+        QString("-s:%1").arg(ublFileName()),
+        QString("-xsl:%1").arg(xslPdf),
+    };
+    
+    const QStringList fop_args {
+        "-fo", "-",
+        "-pdf", path,
+    };
+
+    QProcess *xsl_process = new QProcess;
+    QProcess *fop_process = new QProcess;
+
+	xsl_process->setStandardOutputProcess(fop_process);
+
+    connect(xsl_process, &QProcess::errorOccurred, this, &XRechnung::slotErrorOccurred);
+    connect(fop_process, &QProcess::errorOccurred, this, &XRechnung::slotErrorOccurred);
+    connect(fop_process, &QProcess::finished, this, &XRechnung::slotPdfFopFinished);
+    connect(xsl_process, &QProcess::finished, this, &XRechnung::slotPdfXslFinished);
+
+    qDebug() << "Starting" << "java" << xsl_args;
+    xsl_process->start("java", xsl_args);
+    qDebug() << "Starting" << "fop" << fop_args;
+    fop_process->start("fop", fop_args);
+}
+
+void XRechnung::slotPdfFopFinished(int /*exitCode*/, QProcess::ExitStatus /*exitStatus*/)
+{
+    QProcess *p = qobject_cast<QProcess*>(sender());
+    qDebug() << "PDF export finished.";
+    const QString errorStr = p->readAllStandardError();
+    if (!errorStr.isEmpty()) {
+	    qDebug() << "Stderr while running fop:\n" << errorStr;
+    }
+    p->deleteLater();
+}
+
+void XRechnung::slotPdfXslFinished(int /*exitCode*/, QProcess::ExitStatus /*exitStatus*/)
+{
+    QProcess *p = qobject_cast<QProcess*>(sender());
+    qDebug() << "PDF XSL tranformation finished.";
+    const QString errorStr = p->readAllStandardError();
+    if (!errorStr.isEmpty()) {
+	    qDebug() << "Stderr while running Saxon:" << errorStr;
+    }
+    p->deleteLater();
+}
 
 bool XRechnung::readDomDocument()
 {
